@@ -3,7 +3,7 @@
 ##
 ## Copyright (C) 2006 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006-2007 Tomasz Melcer <liori AT exroot.org>
-## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##                    Stephan Erb <steve-e AT h3c.de>
 ##
@@ -25,16 +25,16 @@
 # FIXME: think if we need caching command list. it may be wrong if there will
 # be entities that often change the list, it may be slow to fetch it every time
 
-import gobject
-import gtk
+from gi.repository import GLib
+from gi.repository import Gtk
 
 import nbxmpp
-from common import gajim
-from common import dataforms
+from gajim.common import app
+from gajim.common import dataforms
 
-import gtkgui_helpers
-import dialogs
-import dataforms_widget
+from gajim import gtkgui_helpers
+from gajim import dialogs
+from gajim import dataforms_widget
 
 class CommandWindow:
     """
@@ -55,10 +55,17 @@ class CommandWindow:
         """
 
         # an account object
-        self.account = gajim.connections[account]
+        self.account = app.connections[account]
         self.jid = jid
         self.commandnode = commandnode
         self.data_form_widget = None
+        self.stage_finish_cb = None
+        self.stage_back_button_cb = None
+        self.stage_forward_button_cb = None
+        self.stage_execute_button_cb = None
+        self.stage_close_button_cb = None
+        self.stage_restart_button_cb = None
+        self.stage_window_delete_cb = None
 
         # retrieving widgets from xml
         self.xml = gtkgui_helpers.get_gtk_builder('adhoc_commands_window.ui')
@@ -91,7 +98,7 @@ class CommandWindow:
             self.data_form_widget.destroy()
         self.data_form_widget = dataforms_widget.DataFormWidget()
         self.data_form_widget.show()
-        self.sending_form_stage_vbox.pack_start(self.data_form_widget)
+        self.sending_form_stage_vbox.pack_start(self.data_form_widget, True, True, 0)
 
         if self.commandnode:
             # Execute command
@@ -109,57 +116,51 @@ class CommandWindow:
 
     # These functions are set up by appropriate stageX methods.
     def stage_finish(self, *anything):
-        pass
-
-    def stage_back_button_clicked(self, *anything):
-        assert False
-
-    def stage_forward_button_clicked(self, *anything):
-        assert False
-
-    def stage_execute_button_clicked(self, *anything):
-        assert False
-
-    def stage_close_button_clicked(self, *anything):
-        assert False
-
-    def stage_restart_button_clicked(self, *anything):
-        assert False
-
-    def stage_adhoc_commands_window_delete_event(self, *anything):
-        assert False
-
-    def do_nothing(self, *anything):
-        return False
+        if self.stage_finish_cb:
+            self.stage_finish_cb(*anything)
 
     # Widget callbacks...
     def on_back_button_clicked(self, *anything):
-        return self.stage_back_button_clicked(*anything)
+        if self.stage_back_button_cb:
+            return self.stage_back_button_cb(*anything)
+        return False
 
     def on_forward_button_clicked(self, *anything):
-        return self.stage_forward_button_clicked(*anything)
+        if self.stage_forward_button_cb:
+            return self.stage_forward_button_cb(*anything)
+        return False
 
     def on_execute_button_clicked(self, *anything):
-        return self.stage_execute_button_clicked(*anything)
+        if self.stage_execute_button_cb:
+            return self.stage_execute_button_cb(*anything)
+        return False
 
     def on_finish_button_clicked(self, *anything):
-        return self.stage_finish_button_clicked(*anything)
+        if self.stage_finish_button_cb:
+            return self.stage_finish_button_cb(*anything)
+        return False
 
     def on_close_button_clicked(self, *anything):
-        return self.stage_close_button_clicked(*anything)
+        if self.stage_close_button_cb:
+            return self.stage_close_button_cb(*anything)
+        return False
 
     def on_restart_button_clicked(self, *anything):
-        return self.stage_restart_button_clicked(*anything)
+        if self.stage_restart_button_cb:
+            return self.stage_restart_button_cb(*anything)
+        return False
 
     def on_adhoc_commands_window_destroy(self, *anything):
         # TODO: do all actions that are needed to remove this object from memory
         self.remove_pulsing()
 
     def on_adhoc_commands_window_delete_event(self, *anything):
-        return self.stage_adhoc_commands_window_delete_event(self.window)
+        if self.stage_window_delete_cb:
+            return self.stage_window_delete_cb(self.window)
+        return False
 
     def __del__(self):
-        print 'Object has been deleted.'
+        print('Object has been deleted.')
 
 # stage 1: waiting for command list
     def stage1(self):
@@ -188,10 +189,10 @@ class CommandWindow:
             self.xml.get_object('retrieving_commands_progressbar'))
 
         # setup the callbacks
-        self.stage_finish = self.stage1_finish
-        self.stage_close_button_clicked = self.stage1_close_button_clicked
-        self.stage_restart_button_clicked = self.stage1_restart_button_clicked
-        self.stage_adhoc_commands_window_delete_event = \
+        self.stage_finish_cb = self.stage1_finish
+        self.stage_close_button_cb = self.stage1_close_button_clicked
+        self.stage_restart_button_cb = self.stage1_restart_button_clicked
+        self.stage_window_delete_cb = \
             self.stage1_adhoc_commands_window_delete_event
 
     def stage1_finish(self):
@@ -236,28 +237,28 @@ class CommandWindow:
         # build the commands list radiobuttons
         first_radio = None
         for (commandnode, commandname) in self.commandlist:
-            radio = gtk.RadioButton(first_radio, label=commandname)
+            radio = Gtk.RadioButton.new_with_label_from_widget(first_radio, commandname)
             radio.connect("toggled", self.on_command_radiobutton_toggled,
                 commandnode)
             if not first_radio:
                 first_radio = radio
                 self.commandnode = commandnode
-            self.command_list_vbox.pack_start(radio, expand=False)
+            self.command_list_vbox.pack_start(radio, False, True, 0)
         self.command_list_vbox.show_all()
 
-        self.stage_finish = self.stage2_finish
-        self.stage_close_button_clicked = self.stage2_close_button_clicked
-        self.stage_restart_button_clicked = self.stage2_restart_button_clicked
-        self.stage_forward_button_clicked = self.stage2_forward_button_clicked
-        self.stage_adhoc_commands_window_delete_event = self.do_nothing
+        self.stage_finish_cb = self.stage2_finish
+        self.stage_close_button_cb = self.stage2_close_button_clicked
+        self.stage_restart_button_cb = self.stage2_restart_button_clicked
+        self.stage_forward_button_cb = self.stage2_forward_button_clicked
+        self.stage_window_delete_cb = None
 
     def stage2_finish(self):
         """
         Remove widgets we created. Not needed when the window is destroyed
         """
-        def remove_widget(widget):
+        def remove_widget(widget, param):
             self.command_list_vbox.remove(widget)
-        self.command_list_vbox.foreach(remove_widget)
+        self.command_list_vbox.foreach(remove_widget, None)
 
     def stage2_close_button_clicked(self, widget):
         self.stage_finish()
@@ -281,8 +282,6 @@ class CommandWindow:
         # close old stage
         self.stage_finish()
 
-        assert isinstance(self.commandnode, unicode)
-
         self.form_status = None
 
         self.stages_notebook.set_current_page(
@@ -298,18 +297,14 @@ class CommandWindow:
 
         self.stage3_submit_form()
 
-        self.stage_finish = self.stage3_finish
-        self.stage_back_button_clicked = self.stage3_back_button_clicked
-        self.stage_forward_button_clicked = self.stage3_forward_button_clicked
-        self.stage_execute_button_clicked = self.stage3_execute_button_clicked
-        self.stage_finish_button_clicked = self.stage3_finish_button_clicked
-        self.stage_close_button_clicked = self.stage3_close_button_clicked
-        self.stage_restart_button_clicked = self.stage3_restart_button_clicked
-        self.stage_adhoc_commands_window_delete_event = \
-            self.stage3_close_button_clicked
-
-    def stage3_finish(self):
-        pass
+        self.stage_finish_cb = None
+        self.stage_back_button_cb = self.stage3_back_button_clicked
+        self.stage_forward_button_cb = self.stage3_forward_button_clicked
+        self.stage_execute_button_cb = self.stage3_execute_button_clicked
+        self.stage_finish_button_cb = self.stage3_finish_button_clicked
+        self.stage_close_button_cb = self.stage3_close_button_clicked
+        self.stage_restart_button_cb = self.stage3_restart_button_clicked
+        self.stage_window_delete_cb = self.stage3_close_button_clicked
 
     def stage3_can_close(self, cb):
         if self.form_status == 'completed':
@@ -321,8 +316,8 @@ class CommandWindow:
             dialog.destroy()
             cb()
 
-        dialog = dialogs.HigDialog(self.window, gtk.MESSAGE_WARNING,
-            gtk.BUTTONS_YES_NO, _('Cancel confirmation'),
+        dialog = dialogs.HigDialog(self.window, Gtk.MessageType.WARNING,
+            Gtk.ButtonsType.YES_NO, _('Cancel confirmation'),
             _('You are in process of executing command. Do you really want to '
             'cancel it?'), on_response_yes=on_yes)
         dialog.popup()
@@ -369,10 +364,8 @@ class CommandWindow:
         if self.data_form_widget.get_data_form():
             df = self.data_form_widget.get_data_form()
             if not df.is_valid():
-                dialogs.ErrorDialog(
-                    _('Invalid Form'),
-                    _('The form is not filled correctly.'),
-                    transient_for=self.window)
+                app.interface.raise_dialog(
+                    'invalid-form', transient_for=self.window)
                 self.data_form_widget.set_sensitive(True)
                 return
             self.data_form_widget.data_form.type_ = 'submit'
@@ -449,12 +442,11 @@ class CommandWindow:
             self.execute_button.hide()
             self.finish_button.hide()
             self.close_button.show()
-            self.stage_adhoc_commands_window_delete_event = \
-                self.stage3_close_button_clicked
+            self.stage_window_delete_cb = self.stage3_close_button_clicked
 
         note = command.getTag('note')
         if note:
-            self.notes_label.set_text(note.getData().decode('utf-8'))
+            self.notes_label.set_text(note.getData())
             self.notes_label.set_no_show_all(False)
             self.notes_label.show()
         else:
@@ -482,10 +474,10 @@ class CommandWindow:
         self.execute_button.set_sensitive(False)
         self.finish_button.set_sensitive(False)
 
-        self.stage_finish = self.do_nothing
-        self.stage_close_button_clicked = self.stage4_close_button_clicked
-        self.stage_restart_button_clicked = self.stage4_restart_button_clicked
-        self.stage_adhoc_commands_window_delete_event = self.do_nothing
+        self.stage_finish_cb = None
+        self.stage_close_button_cb = self.stage4_close_button_clicked
+        self.stage_restart_button_cb = self.stage4_restart_button_clicked
+        self.stage_window_delete_cb = None
 
     def stage4_close_button_clicked(self, widget):
         self.window.destroy()
@@ -512,7 +504,7 @@ class CommandWindow:
             try:
                 errorname = nbxmpp.NS_STANZAS + ' ' + str(errorid)
                 errordesc = nbxmpp.ERRORS[errorname][2]
-                error = errordesc.decode('utf-8')
+                error = errordesc
                 del errorname, errordesc
             except KeyError:        # when stanza doesn't have error description
                 error = _('Service returned an error.')
@@ -534,10 +526,10 @@ class CommandWindow:
 
         self.error_description_label.set_text(error)
 
-        self.stage_finish = self.do_nothing
-        self.stage_close_button_clicked = self.stage5_close_button_clicked
-        self.stage_restart_button_clicked = self.stage5_restart_button_clicked
-        self.stage_adhoc_commands_window_delete_event = self.do_nothing
+        self.stage_finish_cb = None
+        self.stage_close_button_cb = self.stage5_close_button_clicked
+        self.stage_restart_button_cb = self.stage5_restart_button_clicked
+        self.stage_window_delete_cb = None
 
     def stage5_close_button_clicked(self, widget):
         self.window.destroy()
@@ -552,21 +544,21 @@ class CommandWindow:
         progressbar.pulse() method
         """
         assert not self.pulse_id
-        assert isinstance(progressbar, gtk.ProgressBar)
+        assert isinstance(progressbar, Gtk.ProgressBar)
 
         def callback():
             progressbar.pulse()
             return True     # important to keep callback be called back!
 
         # 12 times per second (80 miliseconds)
-        self.pulse_id = gobject.timeout_add(80, callback)
+        self.pulse_id = GLib.timeout_add(80, callback)
 
     def remove_pulsing(self):
         """
         Stop pulsing, useful when especially when removing widget
         """
         if self.pulse_id:
-            gobject.source_remove(self.pulse_id)
+            GLib.source_remove(self.pulse_id)
         self.pulse_id = None
 
 # handling xml stanzas
@@ -610,7 +602,6 @@ class CommandWindow:
         Send the command with data form. Wait for reply
         """
         # create the stanza
-        assert isinstance(self.commandnode, unicode)
         assert action in ('execute', 'prev', 'next', 'complete')
 
         stanza = nbxmpp.Iq(typ='set', to=self.jid)
