@@ -3,7 +3,7 @@
 ##
 ## Copyright (C) 2007 Tomasz Melcer <liori AT exroot.org>
 ##                    Travis Shirk <travis AT pobox.com>
-## Copyright (C) 2007-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
 ##                    Jonathan Schleifer <js-gajim AT webkeks.org>
 ## Copyright (C) 2008-2009 Stephan Erb <steve-e AT h3c.de>
@@ -33,17 +33,19 @@ through ClientCaps objects which are hold by contact instances.
 
 import base64
 import hashlib
+from collections import namedtuple
 
 import logging
 log = logging.getLogger('gajim.c.caps_cache')
 
-from nbxmpp import (NS_XHTML_IM, NS_RECEIPTS, NS_ESESSION, NS_CHATSTATES,
+import nbxmpp
+from nbxmpp import (NS_XHTML_IM, NS_ESESSION, NS_CHATSTATES,
     NS_JINGLE_ICE_UDP, NS_JINGLE_RTP_AUDIO, NS_JINGLE_RTP_VIDEO,
-    NS_JINGLE_FILE_TRANSFER)
+    NS_JINGLE_FILE_TRANSFER_5)
 # Features where we cannot safely assume that the other side supports them
-FEATURE_BLACKLIST = [NS_CHATSTATES, NS_XHTML_IM, NS_RECEIPTS, NS_ESESSION,
+FEATURE_BLACKLIST = [NS_CHATSTATES, NS_XHTML_IM, NS_ESESSION,
     NS_JINGLE_ICE_UDP, NS_JINGLE_RTP_AUDIO, NS_JINGLE_RTP_VIDEO,
-    NS_JINGLE_FILE_TRANSFER]
+    NS_JINGLE_FILE_TRANSFER_5]
 
 # Query entry status codes
 NEW = 0
@@ -56,12 +58,15 @@ FAKED = 3 # allow NullClientCaps to behave as it has a cached item
 ################################################################################
 
 capscache = None
+muc_caps_cache = None
 def initialize(logger):
     """
     Initialize this module
     """
     global capscache
+    global muc_caps_cache
     capscache = CapsCache(logger)
+    muc_caps_cache = MucCapsCache()
 
 def client_supports(client_caps, requested_feature):
     lookup_item = client_caps.get_cache_lookup_strategy()
@@ -94,43 +99,25 @@ def create_suitable_client_caps(node, caps_hash, hash_method, fjid=None):
         client_caps = ClientCaps(caps_hash, node, hash_method)
     return client_caps
 
-def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
+def compute_caps_hash(identities, features, dataforms=None, hash_method='sha-1'):
     """
     Compute caps hash according to XEP-0115, V1.5
 
     dataforms are nbxmpp.DataForms objects as common.dataforms don't allow several
     values without a field type list-multi
     """
-    def sort_identities_func(i1, i2):
-        cat1 = i1['category']
-        cat2 = i2['category']
-        if cat1 < cat2:
-            return -1
-        if cat1 > cat2:
-            return 1
-        type1 = i1.get('type', '')
-        type2 = i2.get('type', '')
-        if type1 < type2:
-            return -1
-        if type1 > type2:
-            return 1
-        lang1 = i1.get('xml:lang', '')
-        lang2 = i2.get('xml:lang', '')
-        if lang1 < lang2:
-            return -1
-        if lang1 > lang2:
-            return 1
-        return 0
+    if dataforms is None:
+        dataforms = []
 
-    def sort_dataforms_func(d1, d2):
-        f1 = d1.getField('FORM_TYPE')
-        f2 = d2.getField('FORM_TYPE')
-        if f1 and f2 and (f1.getValue() < f2.getValue()):
-            return -1
-        return 1
+    def sort_identities_key(i):
+        return (i['category'], i.get('type', ''), i.get('xml:lang', ''))
+
+    def sort_dataforms_key(dataform):
+        f = dataform.getField('FORM_TYPE')
+        return (bool(f), f.getValue())
 
     S = ''
-    identities.sort(cmp=sort_identities_func)
+    identities.sort(key=sort_identities_key)
     for i in identities:
         c = i['category']
         type_ = i.get('type', '')
@@ -140,7 +127,7 @@ def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
     features.sort()
     for f in features:
         S += '%s<' % f
-    dataforms.sort(cmp=sort_dataforms_func)
+    dataforms.sort(key=sort_dataforms_key)
     for dataform in dataforms:
         # fields indexed by var
         fields = {}
@@ -157,12 +144,12 @@ def compute_caps_hash(identities, features, dataforms=[], hash_method='sha-1'):
                 S += '%s<' % value
 
     if hash_method == 'sha-1':
-        hash_ = hashlib.sha1(S)
+        hash_ = hashlib.sha1(S.encode('utf-8'))
     elif hash_method == 'md5':
-        hash_ = hashlib.md5(S)
+        hash_ = hashlib.md5(S.encode('utf-8'))
     else:
         return ''
-    return base64.b64encode(hash_.digest())
+    return base64.b64encode(hash_.digest()).decode('utf-8')
 
 
 ################################################################################
@@ -435,3 +422,65 @@ class CapsCache(object):
         key = (hash_method, hash)
         if key in self.__cache:
             del self.__cache[key]
+
+
+class MucCapsCache:
+
+    DiscoInfo = namedtuple('DiscoInfo', ['identities', 'features', 'data'])
+
+    def __init__(self):
+        self.cache = {}
+
+    def append(self, stanza):
+        jid = stanza.getFrom()
+        identities, features, data = [], [], []
+        query_childs = stanza.getQueryChildren()
+        if not query_childs:
+            from gajim.common import app
+            app.log('gajim.muc').warning('%s returned empty disco info', jid)
+            return
+
+        for child in query_childs:
+            if child.getName() == 'identity':
+                attr = {}
+                for key in child.getAttrs().keys():
+                    attr[key] = child.getAttr(key)
+                identities.append(attr)
+            elif child.getName() == 'feature':
+                features.append(child.getAttr('var'))
+            elif child.getName() == 'x':
+                if child.getNamespace() == nbxmpp.NS_DATA:
+                    data.append(nbxmpp.DataForm(node=child))
+
+        if nbxmpp.NS_MUC not in features:
+            # Not a MUC, dont cache info
+            return
+
+        self.cache[jid] = self.DiscoInfo(identities, features, data)
+
+    def is_cached(self, jid):
+        return jid in self.cache
+
+    def supports(self, jid, feature):
+        if jid in self.cache:
+            if feature in self.cache[jid].features:
+                return True
+        return False
+
+    def has_mam(self, jid):
+        try:
+            if nbxmpp.NS_MAM_2 in self.cache[jid].features:
+                return True
+            if nbxmpp.NS_MAM_1 in self.cache[jid].features:
+                return True
+        except (KeyError, AttributeError):
+            return False
+
+    def get_mam_namespace(self, jid):
+        try:
+            if nbxmpp.NS_MAM_2 in self.cache[jid].features:
+                return nbxmpp.NS_MAM_2
+            elif nbxmpp.NS_MAM_1 in self.cache[jid].features:
+                return nbxmpp.NS_MAM_1
+        except (KeyError, AttributeError):
+            return
