@@ -431,10 +431,8 @@ def get_uf_sub(sub):
         uf_sub = _('From')
     elif sub == 'both':
         uf_sub = _('Both')
-    elif sub is None:
-        uf_sub = _('Unknown')
     else:
-        uf_sub = sub
+        uf_sub = _('Unknown')
 
     return uf_sub
 
@@ -1242,8 +1240,13 @@ def get_accounts_info():
             message = message.strip()
         if message != '':
             single_line += ': ' + message
-        accounts.append({'name': account, 'status_line': single_line,
-                        'show': status, 'message': message})
+        account_label = app.config.get_per(
+            'accounts', account, 'account_label')
+        accounts.append({'name': account,
+                         'account_label': account_label or account,
+                         'status_line': single_line,
+                         'show': status,
+                         'message': message})
     return accounts
 
 
@@ -1626,3 +1629,21 @@ def get_emoticon_theme_path(theme):
     emoticons_user_path = os.path.join(app.MY_EMOTS_PATH, theme)
     if os.path.exists(emoticons_user_path):
         return emoticons_user_path
+
+def add_to_mam_blacklist(jid):
+    config_value = app.config.get('mam_blacklist')
+    if not config_value:
+        config_value = [jid]
+    else:
+        if jid in config_value:
+            return
+        config_value = config_value.split(',')
+        config_value.append(jid)
+    log.warning('Found not-compliant MUC. %s added to MAM Blacklist', jid)
+    app.config.set('mam_blacklist', ','.join(config_value))
+
+def get_mam_blacklist():
+    config_value = app.config.get('mam_blacklist')
+    if not config_value:
+        return []
+    return config_value.split(',')
