@@ -2,7 +2,7 @@
 ## src/common/pep.py
 ##
 ## Copyright (C) 2007 Piotr Gaczkowski <doomhammerng AT gmail.com>
-## Copyright (C) 2007-2017 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2007-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2008 Brendan Taylor <whateley AT gmail.com>
 ##                    Jean-Marie Traissard <jim AT lapin.org>
 ##                    Jonathan Schleifer <js-common.gajim AT webkeks.org>
@@ -215,12 +215,13 @@ LOCATION_DATA = {
         'timestamp':    _('timestamp'),
         'uri':          _('uri')}
 
-import gobject
+from gi.repository import GLib
+
 import logging
 log = logging.getLogger('gajim.c.pep')
 
 import nbxmpp
-from common import gajim
+from gajim.common import app
 
 
 class AbstractPEP(object):
@@ -241,15 +242,16 @@ class AbstractPEP(object):
         self._pep_specific_data, self._retracted = self._extract_info(items)
 
         self._update_contacts(jid, account)
-        if jid == gajim.get_jid_from_account(account):
+        if jid == app.get_jid_from_account(account):
             self._update_account(account)
+        self._on_receive(jid, account)
 
     def _extract_info(self, items):
         '''To be implemented by subclasses'''
         raise NotImplementedError
 
     def _update_contacts(self, jid, account):
-        for contact in gajim.contacts.get_contacts(account, jid):
+        for contact in app.contacts.get_contacts(account, jid):
             if self._retracted:
                 if self.type_ in contact.pep:
                     del contact.pep[self.type_]
@@ -257,21 +259,21 @@ class AbstractPEP(object):
                 contact.pep[self.type_] = self
 
     def _update_account(self, account):
-        acc = gajim.connections[account]
+        acc = app.connections[account]
         if self._retracted:
             if self.type_ in acc.pep:
                 del acc.pep[self.type_]
         else:
             acc.pep[self.type_] = self
 
-    def asPixbufIcon(self):
-        '''SHOULD be implemented by subclasses'''
-        return None
-
     def asMarkupText(self):
         '''SHOULD be implemented by subclasses'''
         return ''
 
+    def _on_receive(self, jid, account):
+        '''SHOULD be implemented by subclasses'''
+        pass
+
 
 class UserMoodPEP(AbstractPEP):
     '''XEP-0107: User Mood'''
@@ -295,18 +297,14 @@ class UserMoodPEP(AbstractPEP):
         retracted = items.getTag('retract') or not 'mood' in mood_dict
         return (mood_dict, retracted)
 
-    def asPixbufIcon(self):
-        assert not self._retracted
-        return gajim.interface.get_pep_icon(self)
-
     def asMarkupText(self):
         assert not self._retracted
         untranslated_mood = self._pep_specific_data['mood']
         mood = self._translate_mood(untranslated_mood)
-        markuptext = '<b>%s</b>' % gobject.markup_escape_text(mood)
+        markuptext = '<b>%s</b>' % GLib.markup_escape_text(mood)
         if 'text' in self._pep_specific_data:
             text = self._pep_specific_data['text']
-            markuptext += ' (%s)' % gobject.markup_escape_text(text)
+            markuptext += ' (%s)' % GLib.markup_escape_text(text)
         return markuptext
 
     def _translate_mood(self, mood):
@@ -338,21 +336,18 @@ class UserTunePEP(AbstractPEP):
             'title' in tune_dict)
         return (tune_dict, retracted)
 
-    def asPixbufIcon(self):
-        return gajim.interface.get_pep_icon(self)
-
     def asMarkupText(self):
         assert not self._retracted
         tune = self._pep_specific_data
 
         artist = tune.get('artist', _('Unknown Artist'))
-        artist = gobject.markup_escape_text(artist)
+        artist = GLib.markup_escape_text(artist)
 
         title = tune.get('title', _('Unknown Title'))
-        title = gobject.markup_escape_text(title)
+        title = GLib.markup_escape_text(title)
 
         source = tune.get('source', _('Unknown Source'))
-        source = gobject.markup_escape_text(source)
+        source = GLib.markup_escape_text(source)
 
         tune_string =  _('<b>"%(title)s"</b> by <i>%(artist)s</i>\n'
                 'from <i>%(source)s</i>') % {'title': title,
@@ -386,10 +381,6 @@ class UserActivityPEP(AbstractPEP):
         retracted = items.getTag('retract') or not 'activity' in activity_dict
         return (activity_dict, retracted)
 
-    def asPixbufIcon(self):
-        assert not self._retracted
-        return gajim.interface.get_pep_icon(self)
-
     def asMarkupText(self):
         assert not self._retracted
         pep = self._pep_specific_data
@@ -403,12 +394,12 @@ class UserActivityPEP(AbstractPEP):
                 subactivity = ACTIVITIES[activity][subactivity]
             activity = ACTIVITIES[activity]['category']
 
-        markuptext = '<b>' + gobject.markup_escape_text(activity)
+        markuptext = '<b>' + GLib.markup_escape_text(activity)
         if subactivity:
-            markuptext += ': ' + gobject.markup_escape_text(subactivity)
+            markuptext += ': ' + GLib.markup_escape_text(subactivity)
         markuptext += '</b>'
         if text:
-            markuptext += ' (%s)' % gobject.markup_escape_text(text)
+            markuptext += ' (%s)' % GLib.markup_escape_text(text)
         return markuptext
 
 
@@ -431,14 +422,14 @@ class UserNicknamePEP(AbstractPEP):
 
     def _update_contacts(self, jid, account):
         nick = '' if self._retracted else self._pep_specific_data
-        for contact in gajim.contacts.get_contacts(account, jid):
+        for contact in app.contacts.get_contacts(account, jid):
             contact.contact_name = nick
 
     def _update_account(self, account):
         if self._retracted:
-            gajim.nicks[account] = gajim.config.get_per('accounts', account, 'name')
+            app.nicks[account] = app.config.get_per('accounts', account, 'name')
         else:
-            gajim.nicks[account] = self._pep_specific_data
+            app.nicks[account] = self._pep_specific_data
 
 
 class UserLocationPEP(AbstractPEP):
@@ -464,12 +455,9 @@ class UserLocationPEP(AbstractPEP):
 
     def _update_account(self, account):
         AbstractPEP._update_account(self, account)
-        con = gajim.connections[account].location_info = \
+        con = app.connections[account].location_info = \
                 self._pep_specific_data
 
-    def asPixbufIcon(self):
-        return gajim.interface.get_pep_icon(self)
-
     def asMarkupText(self):
         assert not self._retracted
         location = self._pep_specific_data
@@ -477,7 +465,7 @@ class UserLocationPEP(AbstractPEP):
 
         for entry in location.keys():
             text = location[entry]
-            text = gobject.markup_escape_text(text)
+            text = GLib.markup_escape_text(text)
             # Translate standart location tag
             tag = LOCATION_DATA.get(entry, entry)
             location_string += '\n<b>%(tag)s</b>: %(text)s' % \
@@ -486,5 +474,49 @@ class UserLocationPEP(AbstractPEP):
         return location_string.strip()
 
 
-SUPPORTED_PERSONAL_USER_EVENTS = [UserMoodPEP, UserTunePEP, UserActivityPEP,
-    UserNicknamePEP, UserLocationPEP]
\ No newline at end of file
+class AvatarNotificationPEP(AbstractPEP):
+    '''XEP-0084: Avatars'''
+
+    type_ = 'avatar-notification'
+    namespace = 'urn:xmpp:avatar:metadata'
+
+    def _extract_info(self, items):
+        self.avatar = None
+        for item in items.getTags('item'):
+            metadata = item.getTag('metadata')
+            if metadata is None:
+                app.log('avatar').warning('Invalid avatar stanza:\n%s', items)
+                break
+            info = item.getTag('metadata').getTag('info')
+            if info is not None:
+                self.avatar = info.getAttrs()
+            break
+
+        return (None, False)
+
+    def _on_receive(self, jid, account):
+        con = app.connections[account]
+        if self.avatar is None:
+            # Remove avatar
+            app.log('avatar').info('Remove (Pubsub): %s', jid)
+            app.contacts.set_avatar(account, jid, None)
+            own_jid = con.get_own_jid().getStripped()
+            app.logger.set_avatar_sha(own_jid, jid, None)
+            app.interface.update_avatar(account, jid)
+        else:
+            sha = app.contacts.get_avatar_sha(account, jid)
+            app.log('avatar').info(
+                'Update (Pubsub): %s %s', jid, self.avatar['id'])
+            if sha == self.avatar['id']:
+                app.log('avatar').info(
+                    'Avatar already known (Pubsub): %s %s',
+                    jid, self.avatar['id'])
+                return
+            app.log('avatar').info('Request (Pubsub): %s', jid)
+            con.get_pubsub_avatar(jid, 'urn:xmpp:avatar:data',
+                                  self.avatar['id'])
+
+
+SUPPORTED_PERSONAL_USER_EVENTS = [
+    UserMoodPEP, UserTunePEP, UserActivityPEP,
+    UserNicknamePEP, UserLocationPEP, AvatarNotificationPEP]
